package main

import (
	"context"
	"net"
	"os"
	"os/signal"
	"time"

	"git.pbiernat.dev/egommerce/basket-service/internal/app"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/config"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/database"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/handler"
)

const (
	defHttpIp   = "0.0.0.0"
	defHttpPort = "8080"
	defDbUrl    = "postgres://postgres:12345678@127.0.0.1:5432/egommerce"
)

func main() {
	if config.ErrLoadingEnvs != nil {
		app.Panicf("Error loading .env file")
	}

	httpAddr := net.JoinHostPort(config.GetEnv("SERVER_IP", defHttpIp), defHttpPort)
	dbConnStr := config.GetEnv("DATABASE_URL", defDbUrl)

	dbc, err := database.Connect(dbConnStr)
	if err != nil {
		app.Panicf("Unable to connect to database: %v\n", err)
	}

	env := &handler.Env{httpAddr, dbc}
	srv := app.NewServer(env)

	go srv.Start()

	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c

	ctx, cancel := context.WithTimeout(context.Background(), time.Second*10)
	defer cancel()

	srv.Shutdown(ctx)
	os.Exit(0)
}
